/* $Id: graphics.c,v 1.1 1997/11/30 22:55:35 wtcorrea Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include "app.h"
#include "gldriver.h"
#include "graphics.h"
#include "my.h"

#define NDEBUG
#include "debug.h"

static void dump_string(char *string)
{
    DEBUG(printf(string));
}

static void dump_value(char *string, float value)
{
    if (string)
	DEBUG(printf(string));

    DEBUG(printf("\t%f\n", value));
}

static void dump_coordinates(char *string, float x, float y, float z)
{
    if (string)
	DEBUG(printf(string));

    DEBUG(printf("\t%f\t%f\t%f\n", x, y, z));
}

void gr_traversal_start(void)
{
    dump_string("TRAVERSAL START\n");
    switch (app.usegl) {
    case USEGL_ON:
	gl_traversal_start();
	break;
    case USEGL_OFF:
	my_traversal_start();
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_traversal_end(void)
{
    dump_string("TRAVERSAL END\n");
    switch (app.usegl) {
    case USEGL_ON:
	gl_traversal_end();
	break;
    case USEGL_OFF:
	my_traversal_end();
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_separator_start(void)
{
    dump_string("SEPARATOR START\n");
    switch (app.usegl) {
    case USEGL_ON:
	gl_separator_start();
	break;
    case USEGL_OFF:
	my_separator_start();
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_separator_end(void)
{
    dump_string("SEPARATOR END\n");
    switch (app.usegl) {
    case USEGL_ON:
	gl_separator_end();
	break;
    case USEGL_OFF:
	my_separator_end();
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_camera(float *position,	/* x,y,z */
     float *orientation,	/* x,y,z,theta */
     float near_distance,
     float far_distance,
     float height_angle)
{
    dump_string("CAMERA\n");
    dump_coordinates("position", position[0], position[1], position[2]);
    dump_coordinates("orientation",
	orientation[0], orientation[1], orientation[2]);
    dump_value("theta", orientation[3]);
    dump_value("near distance", near_distance);
    dump_value("far distance", far_distance);
    dump_value("height angle", height_angle);

    switch (app.usegl) {
    case USEGL_ON:
	gl_camera(position, orientation, near_distance, far_distance,
	    height_angle);
	break;
    case USEGL_OFF:
	my_camera(position, orientation, near_distance, far_distance,
	    height_angle);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_point_light(float intensity,
     float *color,		/* r,g,b */
     float *location)
{				/* x,y,z */
    dump_string("POINT LIGHT\n");
    dump_value("intensity", intensity);
    dump_coordinates("color", color[0], color[1], color[2]);
    dump_coordinates("location", location[0], location[1], location[2]);

    switch (app.usegl) {
    case USEGL_ON:
	gl_point_light(intensity, color, location);
	break;
    case USEGL_OFF:
	my_point_light(intensity, color, location);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_material(float *ambient_color,	/* r,g,b */
     float *diffuse_color,	/* r,g,b */
     float *specular_color,	/* r,g,b */
     float *emissive_color,	/* r,g,b */
     float shininess,
     float transparency)
{
    dump_string("MATERIAL\n");
    dump_coordinates("ambient_color",
	ambient_color[0], ambient_color[1], ambient_color[2]);
    dump_coordinates("diffuse_color",
	diffuse_color[0], diffuse_color[1], diffuse_color[2]);
    dump_coordinates("specular_color",
	specular_color[0], specular_color[1], specular_color[2]);
    dump_coordinates("emissive_color",
	emissive_color[0], emissive_color[1], emissive_color[2]);
    dump_value("shininess", shininess);
    dump_value("transparency", transparency);

    switch (app.usegl) {
    case USEGL_ON:
	gl_material(ambient_color, diffuse_color, specular_color,
	    emissive_color, shininess, transparency);
	break;
    case USEGL_OFF:
	my_material(ambient_color, diffuse_color, specular_color,
	    emissive_color, shininess, transparency);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_scale(float x, float y, float z)
{
    dump_string("SCALE\n");
    dump_coordinates("coords", x, y, z);

    switch (app.usegl) {
    case USEGL_ON:
	gl_scale(x, y, z);
	break;
    case USEGL_OFF:
	my_scale(x, y, z);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_translation(float x, float y, float z)
{
    dump_string("TRANSLATION\n");
    dump_coordinates("coords", x, y, z);

    switch (app.usegl) {
    case USEGL_ON:
	gl_translation(x, y, z);
	break;
    case USEGL_OFF:
	my_translation(x, y, z);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_rotation(float x, float y, float z, float theta)
{
    dump_string("ROTATION\n");
    dump_coordinates("coords", x, y, z);
    dump_value("theta", theta);

    switch (app.usegl) {
    case USEGL_ON:
	gl_rotation(x, y, z, theta);
	break;
    case USEGL_OFF:
	my_rotation(x, y, z, theta);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_label(char *label)
{
    dump_string(label);
    dump_string(" (LABEL)\n");

    switch (app.usegl) {
    case USEGL_ON:
	gl_label(label);
	break;
    case USEGL_OFF:
	my_label(label);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_triangle(
     float *v0,			/* x,y,z */
     float *v1,
     float *v2,
     float *n0,			/* normal for v0 */
     float *n1,
     float *n2)
{
    dump_string("TRIANGLE\n");
    dump_coordinates("v0", v0[0], v0[1], v0[2]);
    dump_coordinates("v1", v1[0], v1[1], v1[2]);
    dump_coordinates("v2", v2[0], v2[1], v2[2]);
    dump_coordinates("n0", n0[0], n0[1], n0[2]);
    dump_coordinates("n1", n1[0], n1[1], n1[2]);
    dump_coordinates("n2", n2[0], n2[1], n2[2]);

    switch (app.usegl) {
    case USEGL_ON:
	gl_triangle(v0, v1, v2, n0, n1, n2);
	break;
    case USEGL_OFF:
	my_triangle(v0, v1, v2, n0, n1, n2);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_cube(float width, float height, float depth)
{
    dump_coordinates("CUBE", width, height, depth);

    switch (app.usegl) {
    case USEGL_ON:
	gl_cube(width, height, depth);
	break;
    case USEGL_OFF:
	my_cube(width, height, depth);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_sphere(float radius)
{
    dump_value("SPHERE", radius);

    switch (app.usegl) {
    case USEGL_ON:
	gl_sphere(radius);
	break;
    case USEGL_OFF:
	my_sphere(radius);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_cone(float bottom_radius, float height)
{
    dump_value("CONE", bottom_radius);
    dump_value("height", height);

    switch (app.usegl) {
    case USEGL_ON:
	gl_cone(bottom_radius, height);
	break;
    case USEGL_OFF:
	my_cone(bottom_radius, height);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}

void gr_cylinder(float radius, float height)
{
    dump_value("CYLINDER", radius);
    dump_value("height", height);

    switch (app.usegl) {
    case USEGL_ON:
	gl_cylinder(radius, height);
	break;
    case USEGL_OFF:
	my_cylinder(radius, height);
	break;
    default:
	fprintf(stderr, "invalid gl mode\n");
	exit(EXIT_FAILURE);
    }
}
