/* $Id: main.c,v 1.1 1997/11/30 22:55:35 wtcorrea Exp $ */

#include <assert.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#include <tkGLx.h>
#include "app.h"

typedef struct {
    char *key;
    int value;
} entry_t;

static int app_init(Tcl_Interp *ti);
static Tcl_CmdProc cb_b1_click;
static Tcl_CmdProc cb_b1_dclick;
static Tcl_CmdProc cb_b1_motion;
static Tcl_CmdProc cb_b1_release;
static Tcl_CmdProc cb_config;
static Tcl_CmdProc cb_motion;
static Tcl_CmdProc cb_redraw;
static Tcl_CmdProc cb_set_model;
static Tcl_VarTraceProc trace_vars;

static entry_t consts[] = {
    "VIEW_POINTS", VIEW_POINTS,
    "VIEW_WIREFRAME", VIEW_WIREFRAME,
    "VIEW_FLAT", VIEW_FLAT,
    "VIEW_GOURAUD", VIEW_GOURAUD,
    "VIEW_PHONG", VIEW_PHONG,
    "USEGL_ON", USEGL_ON,
    "USEGL_OFF", USEGL_OFF,
    "CULLING_ON", CULLING_ON,
    "CULLING_OFF", CULLING_OFF,
    "HEADLIGHT_ON", HEADLIGHT_ON,
    "HEADLIGHT_OFF", HEADLIGHT_OFF,
    "PROJ_PERSPECTIVE", PROJ_PERSPECTIVE,
    "PROJ_ORTHOGONAL", PROJ_ORTHOGONAL,
    NULL, 0
};

static Tcl_HashTable tbl;

static int app_init(Tcl_Interp *ti)
{
    entry_t *entry;
    int new_entry;
    Tcl_HashEntry *hash_entry;

    Tcl_CreateCommand(ti, "app_b1_click", cb_b1_click, NULL, NULL);
    Tcl_CreateCommand(ti, "app_b1_dclick", cb_b1_dclick, NULL, NULL);
    Tcl_CreateCommand(ti, "app_b1_motion", cb_b1_motion, NULL, NULL);
    Tcl_CreateCommand(ti, "app_b1_release", cb_b1_release, NULL, NULL);
    Tcl_CreateCommand(ti, "app_config", cb_config, NULL, NULL);
    Tcl_CreateCommand(ti, "app_motion", cb_motion, NULL, NULL);
    Tcl_CreateCommand(ti, "app_redraw", cb_redraw, NULL, NULL);
    Tcl_CreateCommand(ti, "app_set_model", cb_set_model, NULL, NULL);

    Tcl_TraceVar(ti, "view", TCL_TRACE_WRITES, trace_vars, NULL);
    Tcl_TraceVar(ti, "usegl", TCL_TRACE_WRITES, trace_vars, NULL);
    Tcl_TraceVar(ti, "culling", TCL_TRACE_WRITES, trace_vars, NULL);
    Tcl_TraceVar(ti, "headlight", TCL_TRACE_WRITES, trace_vars, NULL);
    Tcl_TraceVar(ti, "projection", TCL_TRACE_WRITES, trace_vars, NULL);
    Tcl_LinkVar(ti, "redraw", (char *) &app.redraw, TCL_LINK_INT);
    Tcl_LinkVar(ti, "red", (char *) &app.r, TCL_LINK_INT);
    Tcl_LinkVar(ti, "green", (char *) &app.g, TCL_LINK_INT);
    Tcl_LinkVar(ti, "blue", (char *) &app.b, TCL_LINK_INT);

    Tcl_InitHashTable(&tbl, TCL_STRING_KEYS);
    for (entry = consts; entry->key != NULL; entry++) {
	hash_entry = Tcl_CreateHashEntry(&tbl, entry->key, &new_entry);
	Tcl_SetHashValue(hash_entry, entry->value);
    }

    app.model = NULL;
    return TCL_OK;
}

/* Tcl_AppInit() is called from TclMain() after the Tcl interpreter has
 * been created, and before the script file or interactive command loop
 * is entered. */
int Tcl_AppInit(Tcl_Interp *interp)
{
    /* Initialize packages. */

    /* Tcl */
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* Tk */
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* OpenGL */
    if (TkGLx_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* This application */
    if (app_init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    /* Define a startup filename. This file is read in case the program
     * is run interactively. */
    tcl_RcFileName = ".apprc";

    return TCL_OK;
}

int main(int argc, char *argv[])
{
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;
}

static int cb_b1_click(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_b1_click(x, y, s);
    return TCL_OK;
}

static int cb_b1_dclick(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_b1_dclick(x, y, s);
    return TCL_OK;
}

static int cb_b1_motion(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_b1_motion(x, y, s);
    return TCL_OK;
}

static int cb_b1_release(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_b1_release(x, y, s);
    return TCL_OK;
}

static int cb_config(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int w, h;

    (void) cd;
    if (argc != 3) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <w> <h>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &w) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &h) != TCL_OK) {
	return TCL_ERROR;
    }
    app_config(w, h);
    return TCL_OK;
}

static int cb_motion(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    int x, y, s;

    (void) cd;
    if (argc != 4) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <x> <y> <s>", NULL);
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[1], &x) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[2], &y) != TCL_OK) {
	return TCL_ERROR;
    }
    if (Tcl_GetInt(ti, argv[3], &s) != TCL_OK) {
	return TCL_ERROR;
    }
    app_motion(x, y, s);
    return TCL_OK;
}

static int cb_redraw(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 1) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }
    app_redraw();
    return TCL_OK;
}

static char *trace_vars(ClientData cd, Tcl_Interp *ti,
     char *name1, char *name2, int flags)
{
    int value;
    char *value_str;

    (void) cd;
    value_str = Tcl_GetVar2(ti, name1, name2, flags & TCL_GLOBAL_ONLY);
    assert(value_str);
    value = (int) Tcl_GetHashValue(Tcl_FindHashEntry(&tbl, value_str));

    if (strcmp(name1, "view") == 0) {
	assert((value >= VIEW_POINTS) && (value <= VIEW_PHONG));
	app.view = value;
    } else if (strcmp(name1, "usegl") == 0) {
	assert((value == USEGL_ON) || (value == USEGL_OFF));
	app.usegl = value;
    } else if (strcmp(name1, "culling") == 0) {
	assert((value == CULLING_ON) || (value == CULLING_OFF));
	app.culling = value;
    } else if (strcmp(name1, "headlight") == 0) {
	assert((value == HEADLIGHT_ON) || (value == HEADLIGHT_OFF));
	app.headlight = value;
    } else if (strcmp(name1, "projection") == 0) {
	assert((value == PROJ_PERSPECTIVE) || (value == PROJ_ORTHOGONAL));
	app.projection = value;
    } else {
	fprintf(stderr, "unknown variable\n");
	exit(EXIT_FAILURE);
    }
    return NULL;
}

static int cb_set_model(ClientData cd, Tcl_Interp *ti, int argc, char **argv)
{
    (void) cd;
    if (argc != 2) {
	Tcl_AppendResult(ti, "Usage: ", argv[0], " <filename>", NULL);
	return TCL_ERROR;
    }
    app_set_model(argv[1]);
    return TCL_OK;
}
